const fse = require('fs-extra')
const glob = require('glob')
const rollup = require('./service/rollup')
const path = require('path')

const cm = {}
cm['.editEnv'] = require('./ast/editEnv')
// const { editObfus } = require('./ast/obfus')
cm['.toES5'] = require('./service/babel')
cm['.uglify'] = require('./service/uglify')
cm['.iife'] = require('./service/iife')
cm['.obfuscate'] = require('./service/obfuscate')

/**
 * options.outputDir 输出目录，默认为当前项目.dist
 * options.inputDir 输入目录，默认为当前项目src
 * options.outputNum 输出文件数量，默认为1
 * options.env 设置环境，将影响与环境相关代码，默认取process.env.NODE_ENV
 * options.hostname ajax/jsonp请求的hostname
 * options.rollup 对象，rollup打包设置，为空则跳过rollup
 * options.keyMutation 一个对象，key为字典名,value为需要乱序的字典值
 * options.iifeIgnore 对象，不需要生成iife的文件名
 * options.uglify 对象 Minify options
 * options.versionDefault 替换版本hash的原始值
 * options.obfuscate AST混淆
 */
exports.build = async options => {
  if (!options.inputDir) options.inputDir = 'src'
  if (!options.env) options.env = process.env.NODE_ENV || 'dev'

  let fromDir = options.inputDir

  //before rollup
  fromDir = await doPlugin('beforeRollupPlugins', options, fromDir)

  if (options.rollup) {
    Object.values(options.rollup).forEach(n => {
      n.inputOptions.input = path.join(fromDir, n.inputOptions.input)
    })
    //get rollup bundle dest
    let dirs = Object.values(options.rollup).map(n => {
      let folders = n.outputOptions.file.split('/')
      if (folders.length < 2) throw new Error('rollup output file path error!')
      return folders[0]
    })

    let rollupDist = dirs[0]
    if (dirs.every(n => n === rollupDist)) {
      fromDir = rollupDist
    } else {
      throw new Error('all rollup output file must under same folder!')
    }
    fse.emptyDirSync(`${process.cwd()}/${fromDir}`)

    rollup(options).then(async () => {
      console.log(`rollup is ready`)
      afterRollup(options, fromDir)
    })
  } else {
    afterRollup(options, fromDir)
  }
}

const afterRollup = async (options, fromDir) => {
  //after rollup
  fromDir = await doPlugin('afterRollupPlugins', options, fromDir)

  let buildin = ['.editEnv', '.toES5']
  for (let n of buildin) {
    fromDir = await mutate(options, n, fromDir)
  }

  //before uglify
  fromDir = await doPlugin('beforeUglifyPlugins', options, fromDir)

  fromDir = await mutate(options, '.iife', fromDir)
  fromDir = await mutate(options, '.obfuscate', fromDir)

  //uglify
  fromDir = await mutate(options, '.uglify', fromDir)

  fromDir = await doPlugin('afterUglifyPlugins', options, fromDir)

  //final output
  let output = options.outputDir || '.dist'
  let distDir = path.resolve(process.cwd(), output, options.env)
  fse.emptyDirSync(distDir)
  fse.copySync(`${process.cwd()}/${fromDir}`, distDir)
  let now = new Date()
  fse.copySync(
    `${process.cwd()}/${fromDir}`,
    `${distDir}-${(now.getMonth() + 1 + '').padStart(2, '0')}${(now.getDate() + '').padStart(
      2,
      '0'
    )}${(now.getHours() + '').padStart(2, '0')}${(now.getMinutes() + '').padStart(2, '0')}${
      options.versionHash ? '-' + options.versionHash : ''
    }`
  )
  console.log(`${output} is ready`)

  //after final
  await doPlugin('finalPlugins', options, output)
  // editClass()
  // editObfus()
}

const mutate = async (options, toDir, fromDir) => {
  fse.emptyDirSync(`${process.cwd()}/${toDir}`)
  fse.copySync(`${process.cwd()}/${fromDir}`, `${process.cwd()}/${toDir}`)
  let paths = glob.sync(`${process.cwd()}/${toDir}/**/*.js`)

  for (let i = 0; i < paths.length; i++) {
    let base = path.parse(paths[i]).base
    if (cm[toDir]) {
      if (toDir === '.iife' && options.iifeIgnore && options.iifeIgnore[base]) break
      await cm[toDir](paths[i], options, toDir)
    }
  }

  console.log(`${toDir} is ready`)
  return toDir
}

const doPlugin = async (pluginType, options, fromDir) => {
  let plugins = options[pluginType]

  if (plugins && plugins.length > 0) {
    if (plugins.length === 1) {
      fromDir = await plugins[0](options, fromDir)
      console.log(`${fromDir} is ready`)
    } else if (plugins.length > 1) {
      for (let n of plugins) {
        fromDir = await n(options, fromDir)
        console.log(`${fromDir} is ready`)
      }
    }
  }
  return fromDir
}
